/************************************************************
** $Id: mx.c,v 0.4 2003/11/24 18:38:35 bcx Exp bcx $
**
** Copyright Bryan Costales and Perforce Software, Inc. 2003
**
** This code is "open source" as defined by version 1.9 of
** the Open Source Definition from:
**
**	http://www.opensource.org/docs/definition.php.
**
** Entry points defined in this file:
**
** getmx():
** 	Look up MX records for the passed host, and then
**	look up the IP addresses of each found host.
*************************************************************/

# define EXTERN extern
#include "slow.h"

char **
getmx(char *host, char **list)
{
	char buf[BUFSIZ];
	char *cp;
	HEADER *hp;
	QBUF qbuf;
	int len;
	int acnt, qdcnt;
	u_short pref; 	/* not used */
	u_short type;	/* not used */
	u_long ttl;	/* not used */
	struct hostent *hostp;
	char **p, **pp;
	unsigned char *ep, *up;
	struct in_addr in;
	extern int h_errno;

	errno = 0;

	if (host == NULL || *host == '\0')
		return NULL;

	(void)strcpy(buf, host);
	/*
	 * Make sure host ends in a dot to short circuit lookups
	 */
	cp = buf + strlen(buf) - 1;
	if (*cp != '.')
		*(++cp) = '.';
	*(++cp) = '\0';
	/*
	 * Make sure host does not begin with a dot.
	 */
	cp = buf;
	if (*cp == '.')
		++cp;

	/*
	 * The trailing dot above speeds up the lookup.
	 */
	(void)res_init();
	len = res_query(cp, C_IN, T_MX, (u_char *) &qbuf, sizeof(qbuf));
	if (len < 0) 
	{
		(void) fprintf(stderr, "%s: %s(%d): %s: %s\n",
			Prog, __FILE__, __LINE__, 
			cp, hstrerror(h_errno));
		exit(h_errno);
	}

	hp = (HEADER *)&qbuf;
	up = (u_char *)&qbuf + HFIXEDSZ;
	ep = (u_char *)&qbuf + len;

	for (qdcnt = ntohs(hp->qdcount); qdcnt--; up += len + QFIXEDSZ)
	{
		if ((len = dn_skipname(up, ep)) < 0) 
		{
			(void) fprintf(stderr, "%s: %s(%d): %s: %s\n",
				Prog, __FILE__, __LINE__,
				host, hstrerror(h_errno));
			exit(h_errno);
		}
	}

	acnt = ntohs(hp->ancount);
	while (--acnt >= 0 && up < ep)
	{
		if ((len = dn_expand((u_char *)&qbuf, 
			ep, up, (RES_UNC_T) buf, BUFSIZ-1)) < 0)
		{
			break;
		}
		up += len;
		GETSHORT(type, up);
 		up += INT16SZ;
		GETLONG(ttl, up);
		GETSHORT(len, up);
		if (type != T_MX)
		{
			up += len;
			continue;
		}
		GETSHORT(pref, up);
		if ((len = dn_expand((u_char *)&qbuf, 
			ep, up, (RES_UNC_T) buf, BUFSIZ-1)) < 0)
		{
			break;
		}
		up += len;
		if (is_in_list(buf, list) == FALSE)
			list = push_list(buf, list, FALSE);
		hostp = gethostbyname(buf);
		if (hostp == NULL)
		{
			continue;
		}
		for (p = hostp->h_addr_list; *p != 0; p++)
		{
			(void) memcpy(&in.s_addr, *p, sizeof (in.s_addr));
			(void) sprintf(buf, "%s", inet_ntoa(in));
			if (is_in_list(buf, list) == FALSE)
				list = push_list(buf, list, FALSE);
			for (pp = hostp->h_aliases; *pp != 0; pp++)
			{
				sprintf(buf, "%s", *pp);
				if (is_in_list(buf, list) == FALSE)
					list = push_list(buf, list, FALSE);
			}
		}
	}
	return list;
}

/*
** RBL_LOOKUP -- lookup an IP address at an RBL server
**
*/
int
rbl_lookup(char *server, char *ip, int log)
{
	char buf[BUFSIZ];
	char **iplist;
	int ipcnt;
	char **p;
	struct hostent *hostp;
	extern int h_errno;
	int save_retrans;
	int save_retry;

	errno = 0;

	if (ip == NULL || *ip == '\0')
		return EINVAL;
	if (server == NULL || *server == '\0')
		return EINVAL;

	iplist = mkargv(ip, '.', log);
	if (iplist == NULL)
		return errno;

	ipcnt = 0;
	for (p = iplist; *p != NULL; ++p)
		++ipcnt;
	if (ipcnt < 4)
		return EINVAL;
	sprintf(buf, "%s.%s.%s.%s.%s", iplist[3], iplist[2],
		iplist[1], iplist[0], server);

	save_retrans = _res.retrans;
	save_retry = _res.retry;
	_res.retrans = 0;
	_res.retry = 0;

	hostp = gethostbyname(buf);

	_res.retrans = save_retrans;
	_res.retry = save_retry;

	if (hostp == NULL)
		return -h_errno;

	return 0;
}

